@@ -142,6 +142,9 @@ k8s_opts = [
                help=_("The driver that manages VIFs pools for "
                       "Kubernetes Pods"),
                default='noop'),
+    cfg.StrOpt('nodes_subnets_driver',
+               help=_("The driver that manages listing K8s nodes subnet_ids."),
+               default='config'),
     cfg.BoolOpt('port_debug',
                 help=_('Enable port debug to force kuryr port names to be '
                        'set to their corresponding pod names.'),
@@ -179,7 +182,8 @@ k8s_opts = [
     cfg.ListOpt('enabled_handlers',
                 help=_("The comma-separated handlers that should be "
                        "registered for watching in the pipeline."),
-                default=['vif', 'lb', 'lbaasspec']),
+                default=['vif', 'endpoints', 'service', 'kuryrloadbalancer',
+                         'kuryrport']),
     cfg.BoolOpt('controller_ha',
                 help=_('Enable kuryr-controller active/passive HA. Only '
                        'supported in containerized deployments on Kubernetes '
@@ -222,7 +226,7 @@ neutron_defaults = [
                default=None),
     cfg.IntOpt('network_device_mtu',
                help='Default MTU setting for network interface.',
-               default=1500,),
+               default=0,),
     cfg.IntOpt('lbaas_activation_timeout',
                help=_("Time (in seconds) that kuryr controller waits for "
                       "neutron LBaaS to be activated"),
@@ -245,14 +249,6 @@ octavia_defaults = [
                help=_("Define the communication mode between load balanacer "
                       "and its members"),
                default='L3'),
-    cfg.StrOpt('sg_mode',
-               help=_("Define the LBaaS SG policy."),
-               choices=[('create', 'replace the VIP SG with a new one'),
-                        ('update', 'add rules to the existing VIP SG')],
-               default='update',
-               deprecated_for_removal=True,
-               deprecated_reason="enforce_sg_rules option can be used"
-                                 " instead"),
     cfg.BoolOpt('enforce_sg_rules',
                 help=_("Enable the enforcement of SG rules at the LB SG "
                        "in case the LB does not maintain the source IP "
@@ -275,9 +271,11 @@ cache_defaults = [
 ]
 
 nested_vif_driver_opts = [
-    cfg.StrOpt('worker_nodes_subnet',
-               help=_("Neutron subnet ID for k8s worker node vms."),
-               default=''),
+    cfg.ListOpt('worker_nodes_subnets',
+                help=_("Neutron subnet IDs for k8s worker node VMs."),
+                default=[],
+                deprecated_name='worker_nodes_subnet',
+                deprecated_group='pod_vif_nested'),
     cfg.IntOpt('rev_update_attempts',
                help=_("How many time to try to re-update the neutron resource "
                       "when revision has been changed by other thread"),
