@@ -48,6 +48,10 @@ _OCTAVIA_TAGGING_VERSION = 2, 5
 _OCTAVIA_DL_VERSION = 2, 13
 _OCTAVIA_ACL_VERSION = 2, 12
 _OCTAVIA_PROVIDER_VERSION = 2, 6
+_OCTAVIA_SCTP_VERSION = 2, 23
+
+# HTTP Codes raised by Octavia when a Resource already exists
+OKAY_CODES = (409, 500)
 
 
 class LBaaSv2Driver(base.LBaaSDriver):
@@ -60,6 +64,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
         self._octavia_acls = False
         self._octavia_double_listeners = False
         self._octavia_providers = False
+        self._octavia_sctp = False
         # Check if Octavia API supports tagging.
         # TODO(dulek): *Maybe* this can be replaced with
         #         lbaas.get_api_major_version(version=_OCTAVIA_TAGGING_VERSION)
@@ -84,6 +89,9 @@ class LBaaSv2Driver(base.LBaaSDriver):
                         'Octavia resources.', v_str)
         if v >= _OCTAVIA_PROVIDER_VERSION:
             self._octavia_providers = True
+        if v >= _OCTAVIA_SCTP_VERSION:
+            LOG.info('Octavia API supports SCTP protocol.')
+            self._octavia_sctp = True
 
     def double_listeners_supported(self):
         return self._octavia_double_listeners
@@ -91,6 +99,9 @@ class LBaaSv2Driver(base.LBaaSDriver):
     def providers_supported(self):
         return self._octavia_providers
 
+    def sctp_supported(self):
+        return self._octavia_sctp
+
     def get_octavia_version(self):
         lbaas = clients.get_loadbalancer_client()
         region_name = getattr(CONF.neutron, 'region_name', None)
@@ -140,8 +151,9 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'security_groups': security_groups_ids,
             'provider': provider
         }
-        response = self._ensure(self._create_loadbalancer,
-                                self._find_loadbalancer, request)
+
+        response = self._ensure_loadbalancer(request)
+
         if not response:
             # NOTE(ivc): load balancer was present before 'create', but got
             # deleted externally between 'create' and 'find'
@@ -166,11 +178,16 @@ class LBaaSv2Driver(base.LBaaSDriver):
         all_pod_rules = []
         add_default_rules = False
         os_net = clients.get_network_client()
+        sgs = []
 
         if new_sgs:
             sgs = new_sgs
-        else:
+        elif loadbalancer['security_groups']:
             sgs = loadbalancer['security_groups']
+        else:
+            # NOTE(gryf): in case there is no new SG rules and loadbalancer
+            # has the SG removed, just add default ones.
+            add_default_rules = True
 
         # Check if Network Policy allows listener on the pods
         for sg in sgs:
@@ -189,8 +206,9 @@ class LBaaSv2Driver(base.LBaaSDriver):
                     # with or without remote_ip_prefix.
                     if rule.remote_group_id:
                         continue
-                    if (rule.protocol == protocol.lower() and
-                            rule.direction == 'ingress'):
+                    if ((not rule.protocol or
+                         rule.protocol == protocol.lower())
+                            and rule.direction == 'ingress'):
                         # If listener port not in allowed range, skip
                         min_port = rule.port_range_min
                         max_port = rule.port_range_max
@@ -218,7 +236,12 @@ class LBaaSv2Driver(base.LBaaSDriver):
         if CONF.octavia_defaults.enforce_sg_rules:
             vip_port = self._get_vip_port(loadbalancer)
             if vip_port:
-                lb_sg = vip_port.security_group_ids[0]
+                try:
+                    lb_sg = vip_port.security_group_ids[0]
+                except IndexError:
+                    LOG.warning("We still waiting for SG to be created for "
+                                "VIP %s", vip_port)
+                    raise k_exc.ResourceNotReady(listener_id)
         else:
             LOG.debug("Skipping sg update for lb %s", loadbalancer['name'])
             return
@@ -346,21 +369,21 @@ class LBaaSv2Driver(base.LBaaSDriver):
         try:
             result = self._ensure_provisioned(
                 loadbalancer, listener, self._create_listener,
-                self._find_listener, _LB_STS_POLL_SLOW_INTERVAL)
+                self._find_listener, interval=_LB_STS_POLL_SLOW_INTERVAL)
         except os_exc.SDKException:
-            LOG.exception("Listener creation failed, most probably because "
-                          "protocol %(prot)s is not supported",
-                          {'prot': protocol})
+            LOG.exception("Failed when creating listener for loadbalancer "
+                          "%r", loadbalancer['id'])
             return None
 
         # NOTE(maysams): When ovn-octavia provider is used
         # there is no need to set a security group for
         # the load balancer as it wouldn't be enforced.
-        if not CONF.octavia_defaults.enforce_sg_rules:
+        if not CONF.octavia_defaults.enforce_sg_rules and result:
             os_net = clients.get_network_client()
             vip_port = self._get_vip_port(loadbalancer)
-            os_net.update_port(vip_port.id, security_groups=[])
-            loadbalancer['security_groups'] = []
+            if vip_port:
+                os_net.update_port(vip_port.id, security_groups=[])
+                loadbalancer['security_groups'] = []
 
         return result
 
@@ -376,7 +399,10 @@ class LBaaSv2Driver(base.LBaaSDriver):
         # when enforcing sg rules on the lb sg, meaning octavia
         # Amphora provider is configured.
         if CONF.octavia_defaults.enforce_sg_rules:
-            sg_id = self._get_vip_port(loadbalancer).security_group_ids[0]
+            try:
+                sg_id = self._get_vip_port(loadbalancer).security_group_ids[0]
+            except AttributeError:
+                sg_id = None
             if sg_id:
                 rules = os_net.security_group_rules(security_group_id=sg_id,
                                                     description=listener[
@@ -421,6 +447,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
     def ensure_member(self, loadbalancer, pool,
                       subnet_id, ip, port, target_ref_namespace,
                       target_ref_name, listener_port=None):
+        lbaas = clients.get_loadbalancer_client()
         name = ("%s/%s" % (target_ref_namespace, target_ref_name))
         name += ":%s" % port
         member = {
@@ -433,7 +460,8 @@ class LBaaSv2Driver(base.LBaaSDriver):
         }
         result = self._ensure_provisioned(loadbalancer, member,
                                           self._create_member,
-                                          self._find_member)
+                                          self._find_member,
+                                          update=lbaas.update_member)
 
         network_policy = (
             'policy' in CONF.kubernetes.enabled_handlers and
@@ -639,9 +667,9 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
     def _find_member(self, member, loadbalancer):
         lbaas = clients.get_loadbalancer_client()
+        member = dict(member)
         response = lbaas.members(
             member['pool_id'],
-            name=member['name'],
             project_id=member['project_id'],
             subnet_id=member['subnet_id'],
             address=member['ip'],
@@ -650,6 +678,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
         try:
             os_members = next(response)
             member['id'] = os_members.id
+            member['name'] = os_members.name
             if os_members.provisioning_status == 'ERROR':
                 LOG.debug("Releasing Member %s", os_members.id)
                 self.release_member(loadbalancer, member)
@@ -659,33 +688,60 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         return member
 
-    def _ensure(self, create, find, *args):
-        okay_codes = (409, 500)
+    def _ensure(self, create, find, obj, loadbalancer, update=None):
         try:
-            result = create(args[0])
+            result = create(obj)
             LOG.debug("Created %(obj)s", {'obj': result})
             return result
         except os_exc.HttpException as e:
-            if e.status_code not in okay_codes:
+            if e.status_code not in OKAY_CODES:
                 raise
-        result = find(*args)
+        result = find(obj, loadbalancer)
+        # NOTE(maysams): A conflict may happen when a member is
+        # a lefover and a new pod uses the same address. Let's
+        # attempt to udpate the member name if already existent.
+        if result and obj['name'] != result.get('name') and update:
+            update(result['id'], obj['pool_id'], name=obj['name'])
+            result['name'] = obj['name']
+        if result:
+            LOG.debug("Found %(obj)s", {'obj': result})
+        return result
+
+    def _ensure_loadbalancer(self, loadbalancer):
+        try:
+            result = self._create_loadbalancer(loadbalancer)
+            LOG.debug("Created %(obj)s", {'obj': result})
+            return result
+        except os_exc.HttpException as e:
+            if e.status_code not in OKAY_CODES:
+                raise
+        result = self._find_loadbalancer(loadbalancer)
         if result:
             LOG.debug("Found %(obj)s", {'obj': result})
         return result
 
     def _ensure_provisioned(self, loadbalancer, obj, create, find,
-                            interval=_LB_STS_POLL_FAST_INTERVAL):
+                            interval=_LB_STS_POLL_FAST_INTERVAL, **kwargs):
         for remaining in self._provisioning_timer(_ACTIVATION_TIMEOUT,
                                                   interval):
             if not self._wait_for_provisioning(
                     loadbalancer, remaining, interval):
                 return None
             try:
-                result = self._ensure(create, find, obj, loadbalancer)
+                result = self._ensure(
+                    create, find, obj, loadbalancer, **kwargs)
                 if result:
                     return result
             except os_exc.BadRequestException:
                 raise
+            except os_exc.HttpException as e:
+                if e.status_code == 501:
+                    LOG.exception("Listener creation failed, most probably "
+                                  "because protocol %(prot)s is not supported",
+                                  {'prot': obj['protocol']})
+                    return None
+                else:
+                    raise
             except os_exc.SDKException:
                 pass
 
@@ -802,7 +858,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         lb = klb['status']['loadbalancer']
         try:
-            k8s.patch_crd('status/loadbalancer', klb['metadata']['selfLink'],
+            k8s.patch_crd('status/loadbalancer', utils.get_res_link(klb),
                           {'security_groups': sgs})
         except k_exc.K8sResourceNotFound:
             LOG.debug('KuryrLoadBalancer CRD not found %s', lbaas_name)
