@@ -15,6 +15,7 @@
 
 from kuryr.lib import constants as kl_const
 from openstack import exceptions as os_exc
+from oslo_config import cfg
 from oslo_log import log as logging
 
 from kuryr_kubernetes import clients
@@ -28,17 +29,32 @@ from kuryr_kubernetes import os_vif_util as ovu
 
 LOG = logging.getLogger(__name__)
 
+CONF = cfg.CONF
+
 
 class NeutronPodVIFDriver(base.PodVIFDriver):
     """Manages normal Neutron ports to provide VIFs for Kubernetes Pods."""
 
+    def __init__(self):
+        super(NeutronPodVIFDriver, self).__init__()
+
+        self._tag_on_creation = utils.check_tag_on_creation()
+        if self._tag_on_creation:
+            LOG.info('Neutron supports tagging during bulk port creation.')
+        else:
+            LOG.warning('Neutron does not support tagging during bulk '
+                        'port creation. Kuryr will tag resources after '
+                        'port creation.')
+
     def request_vif(self, pod, project_id, subnets, security_groups):
         os_net = clients.get_network_client()
 
         rq = self._get_port_request(pod, project_id, subnets, security_groups)
         port = os_net.create_port(**rq)
-        utils.tag_neutron_resources([port])
 
+        self._check_port_binding([port])
+        if not self._tag_on_creation:
+            utils.tag_neutron_resources([port])
         return ovu.neutron_to_osvif_vif(port.binding_vif_type, port, subnets)
 
     def request_vifs(self, pod, project_id, subnets, security_groups,
@@ -54,7 +70,6 @@ class NeutronPodVIFDriver(base.PodVIFDriver):
         except os_exc.SDKException:
             LOG.exception("Error creating bulk ports: %s", bulk_port_rq)
             raise
-        utils.tag_neutron_resources(ports)
 
         vif_plugin = ports[0].binding_vif_type
 
@@ -66,6 +81,9 @@ class NeutronPodVIFDriver(base.PodVIFDriver):
             port_info = os_net.get_port(ports[0].id)
             vif_plugin = port_info.binding_vif_type
 
+        self._check_port_binding(ports)
+        if not self._tag_on_creation:
+            utils.tag_neutron_resources(ports)
         vifs = []
         for port in ports:
             vif = ovu.neutron_to_osvif_vif(vif_plugin, port, subnets)
@@ -75,7 +93,7 @@ class NeutronPodVIFDriver(base.PodVIFDriver):
     def release_vif(self, pod, vif, project_id=None, security_groups=None):
         clients.get_network_client().delete_port(vif.id)
 
-    def activate_vif(self, pod, vif):
+    def activate_vif(self, vif):
         if vif.active:
             return
 
@@ -93,10 +111,10 @@ class NeutronPodVIFDriver(base.PodVIFDriver):
 
     def update_vif_sgs(self, pod, security_groups):
         os_net = clients.get_network_client()
-        pod_state = utils.get_pod_state(pod)
-        if pod_state:
+        vifs = utils.get_vifs(pod)
+        if vifs:
             # NOTE(ltomasbo): It just updates the default_vif security group
-            port_id = pod_state.vifs[constants.DEFAULT_IFNAME].id
+            port_id = vifs[constants.DEFAULT_IFNAME].id
             os_net.update_port(port_id, security_groups=list(security_groups))
 
     def _get_port_request(self, pod, project_id, subnets, security_groups,
@@ -123,4 +141,17 @@ class NeutronPodVIFDriver(base.PodVIFDriver):
         if security_groups:
             port_req_body['security_groups'] = security_groups
 
+        if self._tag_on_creation:
+            tags = CONF.neutron_defaults.resource_tags
+            if tags:
+                port_req_body['tags'] = tags
+
         return port_req_body
+
+    def _check_port_binding(self, ports):
+        if ports[0].binding_vif_type == "binding_failed":
+            for port in ports:
+                clients.get_network_client().delete_port(port.id)
+            LOG.error("Binding failed error for ports: %s."
+                      " Please check Neutron for errors.", ports)
+            raise k_exc.ResourceNotReady(ports)
