@@ -13,6 +13,7 @@
 #    under the License.
 
 from kuryr.lib import constants as kl_const
+from oslo_config import cfg
 from oslo_log import log as logging
 
 from kuryr_kubernetes import clients
@@ -21,8 +22,10 @@ from kuryr_kubernetes import constants
 from kuryr_kubernetes.controller.drivers import neutron_vif
 from kuryr_kubernetes.controller.drivers import utils as c_utils
 from kuryr_kubernetes import os_vif_util as ovu
+from kuryr_kubernetes import utils
 
 LOG = logging.getLogger(__name__)
+CONF = cfg.CONF
 
 
 def sriov_make_resource(prefix, res_name):
@@ -57,11 +60,12 @@ class SriovVIFDriver(neutron_vif.NeutronPodVIFDriver):
 
         port = os_net.create_port(**rq)
         self._check_port_binding([port])
-        c_utils.tag_neutron_resources([port])
+        if not self._tag_on_creation:
+            c_utils.tag_neutron_resources([port])
         vif = ovu.neutron_to_osvif_vif(vif_plugin, port, subnets)
         vif.physnet = physnet
         vif.pod_name = pod_name
-        vif.pod_link = pod['metadata']['selfLink']
+        vif.pod_link = utils.get_res_link(pod)
 
         LOG.debug("{} vifs are available for the pod {}".format(
             amount, pod_name))
@@ -178,4 +182,9 @@ class SriovVIFDriver(neutron_vif.NeutronPodVIFDriver):
         if security_groups:
             port_req_body['security_groups'] = security_groups
 
+        if self._tag_on_creation:
+            tags = CONF.neutron_defaults.resource_tags
+            if tags:
+                port_req_body['tags'] = tags
+
         return port_req_body
