@@ -23,6 +23,7 @@ from kuryr_kubernetes.controller.drivers import base as drivers
 from kuryr_kubernetes.controller.drivers import utils as driver_utils
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.handlers import k8s_base
+from kuryr_kubernetes import utils
 
 LOG = logging.getLogger(__name__)
 KURYRPORT_URI = constants.K8S_API_CRD_NAMESPACES + '/{ns}/kuryrports/{crd}'
@@ -51,7 +52,7 @@ class KuryrPortHandler(k8s_base.ResourceEventHandler):
             specific_driver='multi_pool')
         self._drv_vif_pool.set_vif_driver()
         self._drv_multi_vif = drivers.MultiVIFDriver.get_enabled_drivers()
-        if self._is_network_policy_enabled():
+        if driver_utils.is_network_policy_enabled():
             self._drv_lbaas = drivers.LBaaSDriver.get_instance()
             self._drv_svc_sg = (drivers.ServiceSecurityGroupsDriver
                                 .get_instance())
@@ -116,7 +117,7 @@ class KuryrPortHandler(k8s_base.ResourceEventHandler):
                 except k_exc.K8sClientException:
                     raise k_exc.ResourceNotReady(pod['metadata']['name'])
 
-                if self._is_network_policy_enabled():
+                if driver_utils.is_network_policy_enabled():
                     crd_pod_selectors = self._drv_sg.create_sg_rules(pod)
                     if oslo_cfg.CONF.octavia_defaults.enforce_sg_rules:
                         services = driver_utils.get_services()
@@ -142,7 +143,11 @@ class KuryrPortHandler(k8s_base.ResourceEventHandler):
                 raise
             return
 
-        if 'deletionTimestamp' not in pod['metadata']:
+        # FIXME(dulek): hostNetwork condition can be removed once we know we
+        #               won't upgrade from version creating ports for host
+        #               networking pods.
+        if ('deletionTimestamp' not in pod['metadata'] and
+                not driver_utils.is_host_network(pod)):
             # NOTE(gryf): Ignore deleting KuryrPort, since most likely it was
             # removed manually, while we need vifs for corresponding pod
             # object which apperantely is still running.
@@ -183,7 +188,7 @@ class KuryrPortHandler(k8s_base.ResourceEventHandler):
             vif = objects.base.VersionedObject.obj_from_primitive(data['vif'])
             self._drv_vif_pool.release_vif(pod, vif, project_id,
                                            security_groups)
-        if (self._is_network_policy_enabled() and crd_pod_selectors and
+        if (driver_utils.is_network_policy_enabled() and crd_pod_selectors and
                 oslo_cfg.CONF.octavia_defaults.enforce_sg_rules):
             services = driver_utils.get_services()
             self._update_services(services, crd_pod_selectors, project_id)
@@ -221,8 +226,17 @@ class KuryrPortHandler(k8s_base.ResourceEventHandler):
             return False
 
         # Request the default interface of pod
-        main_vif = self._drv_vif_pool.request_vif(
-            pod, project_id, subnets, security_groups)
+        try:
+            main_vif = self._drv_vif_pool.request_vif(pod, project_id,
+                                                      subnets,
+                                                      security_groups)
+        except os_exc.ResourceNotFound:
+            # NOTE(gryf): It might happen, that between getting security
+            # groups above and requesting VIF, network policy is deleted,
+            # hence we will get 404 from OpenStackSDK. Let's retry, to refresh
+            # information regarding SG.
+            LOG.warning("SG not found during VIF requesting. Retrying.")
+            raise k_exc.ResourceNotReady(pod['metadata']['name'])
 
         if not main_vif:
             pod_name = pod['metadata']['name']
@@ -261,14 +275,9 @@ class KuryrPortHandler(k8s_base.ResourceEventHandler):
             vif_dict[ifname] = {'default': data['default'],
                                 'vif': data['vif'].obj_to_primitive()}
 
-        self.k8s.patch_crd('status', kuryrport_crd['metadata']['selfLink'],
+        self.k8s.patch_crd('status', utils.get_res_link(kuryrport_crd),
                            {'vifs': vif_dict})
 
-    def _is_network_policy_enabled(self):
-        enabled_handlers = oslo_cfg.CONF.kubernetes.enabled_handlers
-        svc_sg_driver = oslo_cfg.CONF.kubernetes.service_security_groups_driver
-        return ('policy' in enabled_handlers and svc_sg_driver == 'policy')
-
     def _update_services(self, services, crd_pod_selectors, project_id):
         for service in services.get('items'):
             if not driver_utils.service_matches_affected_pods(
