@@ -20,6 +20,7 @@ from kuryr_kubernetes import clients
 from kuryr_kubernetes import config
 from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes.controller.drivers import base as drv_base
+from kuryr_kubernetes.controller.drivers import utils as driver_utils
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.handlers import k8s_base
 from kuryr_kubernetes import utils
@@ -45,6 +46,10 @@ class ServiceHandler(k8s_base.ResourceEventHandler):
         self._drv_subnets = drv_base.ServiceSubnetsDriver.get_instance()
         self._drv_sg = drv_base.ServiceSecurityGroupsDriver.get_instance()
 
+    def _bump_network_policies(self, svc):
+        if driver_utils.is_network_policy_enabled():
+            driver_utils.bump_networkpolicies(svc['metadata']['namespace'])
+
     def on_present(self, service):
         reason = self._should_ignore(service)
         if reason:
@@ -62,6 +67,9 @@ class ServiceHandler(k8s_base.ResourceEventHandler):
 
         if loadbalancer_crd is None:
             try:
+                # Bump all the NPs in the namespace to force SG rules
+                # recalculation.
+                self._bump_network_policies(service)
                 self.create_crd_spec(service)
             except k_exc.K8sNamespaceTerminating:
                 LOG.warning('Namespace %s is being terminated, ignoring '
@@ -111,6 +119,9 @@ class ServiceHandler(k8s_base.ResourceEventHandler):
 
         klb_crd_path = (f"{k_const.K8S_API_CRD_NAMESPACES}/"
                         f"{svc_namespace}/kuryrloadbalancers/{svc_name}")
+        # Bump all the NPs in the namespace to force SG rules
+        # recalculation.
+        self._bump_network_policies(service)
         try:
             k8s.delete(klb_crd_path)
         except k_exc.K8sResourceNotFound:
@@ -283,6 +294,9 @@ class EndpointsHandler(k8s_base.ResourceEventHandler):
         else:
             self._update_crd_spec(loadbalancer_crd, endpoints)
 
+    def on_deleted(self, endpoints, *args, **kwargs):
+        self._remove_endpoints(endpoints)
+
     def _has_pods(self, endpoints):
         ep_subsets = endpoints.get('subsets', [])
         if not ep_subsets:
@@ -382,3 +396,19 @@ class EndpointsHandler(k8s_base.ResourceEventHandler):
             raise
 
         return True
+
+    def _remove_endpoints(self, endpoints):
+        kubernetes = clients.get_kubernetes_client()
+        lb_name = endpoints['metadata']['name']
+        try:
+            kubernetes.patch_crd('spec',
+                                 utils.get_klb_crd_path(endpoints),
+                                 'endpointSlices',
+                                 action='remove')
+        except k_exc.K8sResourceNotFound:
+            LOG.debug('KuryrLoadBalancer CRD not found %s', lb_name)
+        except k_exc.K8sUnprocessableEntity:
+            LOG.warning('KuryrLoadBalancer %s modified, ignoring.', lb_name)
+        except k_exc.K8sClientException:
+            LOG.exception('Error updating KuryrLoadBalancer CRD %s', lb_name)
+            raise
