@@ -24,6 +24,8 @@ from kuryr_kubernetes.controller.drivers import multi_vif
 from kuryr_kubernetes.controller.handlers import kuryrport
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.tests import base as test_base
+from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
+from kuryr_kubernetes import utils
 
 
 CONF = cfg.CONF
@@ -40,22 +42,23 @@ class TestKuryrPortHandler(test_base.TestCase):
         self._pod_version = mock.sentinel.pod_version
         self._pod_link = mock.sentinel.pod_link
         self._kp_version = mock.sentinel.kp_version
-        self._kp_link = mock.sentinel.kp_link
         self._kp_namespace = mock.sentinel.namespace
         self._kp_uid = mock.sentinel.kp_uid
         self._kp_name = 'pod1'
 
-        self._pod = {'metadata': {'resourceVersion': self._pod_version,
-                                  'selfLink': self._pod_link,
+        self._pod = {'apiVersion': 'v1',
+                     'kind': 'Pod',
+                     'metadata': {'resourceVersion': self._pod_version,
                                   'name': self._kp_name,
                                   'deletionTimestamp': mock.sentinel.date,
                                   'namespace': self._kp_namespace},
                      'spec': {'nodeName': self._host}}
 
         self._kp = {
+            'apiVersion': 'openstack.org/v1',
+            'kind': 'KuryrPort',
             'metadata': {
                 'resourceVersion': self._kp_version,
-                'selfLink': self._kp_link,
                 'name': self._kp_name,
                 'namespace': self._kp_namespace,
                 'labels': {
@@ -68,6 +71,7 @@ class TestKuryrPortHandler(test_base.TestCase):
             },
             'status': {'vifs': {}}
         }
+
         self._vif1 = os_obj.vif.VIFBase()
         self._vif2 = os_obj.vif.VIFBase()
         self._vif1.active = False
@@ -87,6 +91,7 @@ class TestKuryrPortHandler(test_base.TestCase):
         self._pod_uri = (f"{constants.K8S_API_NAMESPACES}"
                          f"/{self._kp['metadata']['namespace']}/pods/"
                          f"{self._kp['metadata']['name']}")
+        self.useFixture(k_fix.MockNetworkClient())
         self._driver = multi_vif.NoopMultiVIFDriver()
 
     @mock.patch('kuryr_kubernetes.controller.handlers.kuryrport.'
@@ -304,8 +309,8 @@ class TestKuryrPortHandler(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.vif_pool.MultiVIFPool.'
                 'activate_vif')
     @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
-    @mock.patch('kuryr_kubernetes.controller.handlers.kuryrport.'
-                'KuryrPortHandler._is_network_policy_enabled')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'is_network_policy_enabled')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.MultiVIFDriver.'
                 'get_enabled_drivers')
     def test_on_present_np(self, ged, is_np_enabled, get_k8s_client,
@@ -389,8 +394,8 @@ class TestKuryrPortHandler(test_base.TestCase):
                 'ServiceSecurityGroupsDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base.LBaaSDriver.'
                 'get_instance')
-    @mock.patch('kuryr_kubernetes.controller.handlers.kuryrport.'
-                'KuryrPortHandler._is_network_policy_enabled')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'is_network_policy_enabled')
     @mock.patch('kuryr_kubernetes.controller.drivers.vif_pool.MultiVIFPool.'
                 'release_vif')
     @mock.patch('kuryr_kubernetes.controller.drivers.default_security_groups.'
@@ -564,6 +569,37 @@ class TestKuryrPortHandler(test_base.TestCase):
                                             mock.sentinel.subnets,
                                             self._security_groups)
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.vif_pool.MultiVIFPool.'
+                'request_vif')
+    @mock.patch('kuryr_kubernetes.controller.drivers.default_subnet.'
+                'DefaultPodSubnetDriver.get_subnets')
+    @mock.patch('kuryr_kubernetes.controller.drivers.default_security_groups.'
+                'DefaultPodSecurityGroupsDriver.get_security_groups')
+    @mock.patch('kuryr_kubernetes.controller.drivers.default_project.'
+                'DefaultPodProjectDriver.get_project')
+    @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base.MultiVIFDriver.'
+                'get_enabled_drivers')
+    def test_get_vifs_resource_not_found(self, ged, k8s, get_project, get_sg,
+                                         get_subnets, request_vif):
+        ged.return_value = [self._driver]
+        kp = kuryrport.KuryrPortHandler()
+        kp.k8s.get.return_value = self._pod
+        get_sg.return_value = self._security_groups
+        get_project.return_value = self._project_id
+        get_subnets.return_value = mock.sentinel.subnets
+        request_vif.side_effect = os_exc.ResourceNotFound()
+
+        self.assertRaises(k_exc.ResourceNotReady, kp.get_vifs, self._kp)
+
+        kp.k8s.get.assert_called_once_with(self._pod_uri)
+        get_project.assert_called_once_with(self._pod)
+        get_sg.assert_called_once_with(self._pod, self._project_id)
+        get_subnets.assert_called_once_with(self._pod, self._project_id)
+        request_vif.assert_called_once_with(self._pod, self._project_id,
+                                            mock.sentinel.subnets,
+                                            self._security_groups)
+
     @mock.patch('kuryr_kubernetes.controller.handlers.kuryrport.'
                 'KuryrPortHandler._update_kuryrport_crd')
     @mock.patch('kuryr_kubernetes.controller.drivers.vif_pool.MultiVIFPool.'
@@ -663,43 +699,11 @@ class TestKuryrPortHandler(test_base.TestCase):
         vif1 = self._vif1.obj_to_primitive()
         vif2 = self._vif2.obj_to_primitive()
 
-        kp.k8s.patch_crd.assert_called_once_with(
-            'status', self._kp_link, {'vifs': {'eth0': {'default': True,
-                                                        'vif': vif1},
-                                               'eth1': {'default': False,
-                                                        'vif': vif2}}})
-
-    @mock.patch('kuryr_kubernetes.clients.get_kubernetes_client')
-    @mock.patch('kuryr_kubernetes.controller.drivers.base.MultiVIFDriver.'
-                'get_enabled_drivers')
-    def test_is_network_policy_enabled(self, ged, k8s):
-        ged.return_value = [self._driver]
-        kp = kuryrport.KuryrPortHandler()
-
-        CONF.set_override('enabled_handlers', ['fake_handler'],
-                          group='kubernetes')
-        CONF.set_override('service_security_groups_driver', 'foo',
-                          group='kubernetes')
-
-        self.assertFalse(kp._is_network_policy_enabled())
-
-        CONF.set_override('enabled_handlers', ['policy'],
-                          group='kubernetes')
-        CONF.set_override('service_security_groups_driver', 'foo',
-                          group='kubernetes')
-
-        self.assertFalse(kp._is_network_policy_enabled())
-
-        CONF.set_override('enabled_handlers', ['policy'],
-                          group='kubernetes')
-        self.addCleanup(CONF.clear_override, 'enabled_handlers',
-                        group='kubernetes')
-        CONF.set_override('service_security_groups_driver', 'policy',
-                          group='kubernetes')
-        self.addCleanup(CONF.clear_override, 'service_security_groups_driver',
-                        group='kubernetes')
-
-        self.assertTrue(kp._is_network_policy_enabled())
+        arg = {'vifs': {'eth0': {'default': True, 'vif': vif1},
+                        'eth1': {'default': False, 'vif': vif2}}}
+        kp.k8s.patch_crd.assert_called_once_with('status',
+                                                 utils.get_res_link(self._kp),
+                                                 arg)
 
     @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
                 'service_matches_affected_pods')
