@@ -10,7 +10,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import ipaddress
 import random
+import re
 import socket
 import time
 
@@ -71,6 +73,84 @@ MEMOIZE_NODE = cache.get_memoization_decorator(
     CONF, nodes_cache_region, "nodes_caching")
 cache.configure_cache_region(CONF, nodes_cache_region)
 
+RESOURCE_MAP = {'Endpoints': 'endpoints',
+                'KuryrLoadBalancer': 'kuryrloadbalancers',
+                'KuryrNet': 'kuryrnets',
+                'KuryrNetPolicy': 'kuryrnetpolicies',
+                'KuryrNetwork': 'kuryrnetworks',
+                'KuryrNetworkPolicy': 'kuryrnetworkpolicies',
+                'KuryrPort': 'kuryrports',
+                'Namespace': 'namespaces',
+                'NetworkPolicy': 'networkpolicies',
+                'Node': 'nodes',
+                'Pod': 'pods',
+                'Service': 'services',
+                'Machine': 'machines'}
+API_RE = re.compile(r'v\d+')
+
+
+def get_klb_crd_path(obj):
+    """Return klb crd path from provided resource"""
+    namespace = obj['metadata']['namespace']
+    lb_name = obj['metadata']['name']
+
+    return (f"{constants.K8S_API_CRD_NAMESPACES}/"
+            f"{namespace}/"
+            f"kuryrloadbalancers/"
+            f"{lb_name}")
+
+
+def get_res_link(obj):
+    """Return selfLink equivalent for provided resource"""
+    # First try, if we still have it
+    try:
+        return obj['metadata']['selfLink']
+    except KeyError:
+        pass
+
+    # If not, let's proceed with the path assembling.
+    try:
+        res_type = RESOURCE_MAP[obj['kind']]
+    except KeyError:
+        LOG.error('Unknown resource kind: %s', obj.get('kind'))
+        raise
+
+    namespace = ''
+    if obj['metadata'].get('namespace'):
+        namespace = f"/namespaces/{obj['metadata']['namespace']}"
+
+    try:
+        api = f"/apis/{obj['apiVersion']}"
+        if API_RE.match(obj['apiVersion']):
+            api = f"/api/{obj['apiVersion']}"
+    except KeyError:
+        LOG.error("Object doesn't have an apiVersion available: %s", obj)
+        raise
+
+    return f"{api}{namespace}/{res_type}/{obj['metadata']['name']}"
+
+
+def get_api_ver(path):
+    """Get apiVersion out of resource path.
+
+    Path usually is something simillar to:
+
+        /api/v1/namespaces/default/pods/pod-5bb648d658-55n76
+
+    in case of core resources, and:
+
+        /apis/openstack.org/v1/namespaces/default/kuryrloadbalancers/lb-324
+
+    in case of custom resoures.
+    """
+    if path.startswith('/api/'):
+        return path.split('/')[2]
+
+    if path.startswith('/apis/'):
+        return '/'.join(path.split('/')[2:4])
+
+    raise ValueError('Provided path is not Kubernetes api path: %s', path)
+
 
 def utf8_json_decoder(byte_data):
     """Deserializes the bytes into UTF-8 encoded JSON.
@@ -189,7 +269,7 @@ def get_leader_name():
 
 
 @MEMOIZE_NODE
-def get_nodes_ips():
+def get_nodes_ips(node_subnets):
     """Get the IPs of the trunk ports associated to the deployment."""
     trunk_ips = []
     os_net = clients.get_network_client()
@@ -201,7 +281,8 @@ def get_nodes_ips():
         # part of the kuryr deployment
         ports = os_net.ports(status='ACTIVE')
     for port in ports:
-        if port.trunk_details:
+        if (port.trunk_details and port.fixed_ips and
+                port.fixed_ips[0]['subnet_id'] in node_subnets):
             trunk_ips.append(port.fixed_ips[0]['ip_address'])
     return trunk_ips
 
@@ -230,17 +311,55 @@ def get_subnet_cidr(subnet_id):
     return subnet_obj.cidr
 
 
+def get_subnet_id(**filters):
+    os_net = clients.get_network_client()
+    subnets = os_net.subnets(**filters)
+
+    try:
+        return next(subnets).id
+    except StopIteration:
+        return None
+
+
 @MEMOIZE
-def get_subnetpool_version(subnetpool_id):
+def get_subnets_id_cidrs(subnet_ids):
+    os_net = clients.get_network_client()
+    subnets = os_net.subnets()
+    cidrs = [(subnet.id, subnet.cidr) for subnet in subnets
+             if subnet.id in subnet_ids]
+    if len(cidrs) != len(subnet_ids):
+        existing = {subnet.id for subnet in subnets}
+        missing = set(subnet_ids) - existing
+        LOG.exception("CIDRs of subnets %s not found!", missing)
+        raise os_exc.ResourceNotFound()
+    return cidrs
+
+
+def get_subnets_cidrs(subnet_ids):
+    return [x[1] for x in get_subnets_id_cidrs(subnet_ids)]
+
+
+@MEMOIZE
+def _get_subnetpool(subnetpool_id):
     os_net = clients.get_network_client()
     try:
         subnetpool_obj = os_net.get_subnet_pool(subnetpool_id)
     except os_exc.ResourceNotFound:
         LOG.exception("Subnetpool %s not found!", subnetpool_id)
         raise
+    return subnetpool_obj
+
+
+def get_subnetpool_version(subnetpool_id):
+    subnetpool_obj = _get_subnetpool(subnetpool_id)
     return subnetpool_obj.ip_version
 
 
+def get_subnetpool_cidrs(subnetpool_id):
+    subnetpool_obj = _get_subnetpool(subnetpool_id)
+    return subnetpool_obj.prefixes
+
+
 def extract_pod_annotation(annotation):
     obj = objects.base.VersionedObject.obj_from_primitive(annotation)
     # FIXME(dulek): This is code to maintain compatibility with Queens. We can
@@ -308,7 +427,7 @@ def set_lbaas_spec(service, lbaas_spec):
         LOG.debug("Setting LBaaSServiceSpec annotation: %r", lbaas_spec)
         annotation = jsonutils.dumps(lbaas_spec.obj_to_primitive(),
                                      sort_keys=True)
-    svc_link = service['metadata']['selfLink']
+    svc_link = get_res_link(service)
     ep_link = get_endpoints_link(service)
     k8s = clients.get_kubernetes_client()
 
@@ -356,13 +475,13 @@ def set_lbaas_state(endpoints, lbaas_state):
         annotation = jsonutils.dumps(lbaas_state.obj_to_primitive(),
                                      sort_keys=True)
     k8s = clients.get_kubernetes_client()
-    k8s.annotate(endpoints['metadata']['selfLink'],
+    k8s.annotate(get_res_link(endpoints),
                  {constants.K8S_ANNOTATION_LBAAS_STATE: annotation},
                  resource_version=endpoints['metadata']['resourceVersion'])
 
 
 def get_endpoints_link(service):
-    svc_link = service['metadata']['selfLink']
+    svc_link = get_res_link(service)
     link_parts = svc_link.split('/')
 
     if link_parts[-2] != 'services':
@@ -374,7 +493,7 @@ def get_endpoints_link(service):
 
 
 def get_service_link(endpoints):
-    endpoints_link = endpoints['metadata']['selfLink']
+    endpoints_link = get_res_link(endpoints)
     link_parts = endpoints_link.split('/')
 
     if link_parts[-2] != 'endpoints':
@@ -388,7 +507,7 @@ def get_service_link(endpoints):
 def has_port_changes(service, loadbalancer_crd):
     if not loadbalancer_crd:
         return False
-    link = service['metadata']['selfLink']
+    link = get_res_link(service)
     svc_port_set = service['spec'].get('ports')
 
     for port in svc_port_set:
@@ -441,3 +560,74 @@ def clean_lb_crd_status(loadbalancer_name):
         LOG.exception('Error updating KuryrLoadbalancer CRD %s',
                       name)
         raise
+
+
+def is_kubernetes_default_resource(obj):
+    """Check if Object is a resource associated to the API
+
+    Verifies if the Object is on the default namespace
+    and has the name kubernetes. Those name and namespace
+    are given to Kubernetes Service and Endpoints for the API.
+
+    :param obj: Kubernetes object dict
+    :returns: True if is default resource for the API, false
+              otherwise.
+    """
+    return (obj['metadata']['name'] == 'kubernetes' and
+            obj['metadata']['namespace'] == 'default')
+
+
+def get_pod_by_ip(pod_ip, namespace=None):
+    k8s = clients.get_kubernetes_client()
+    pod = {}
+    try:
+        if namespace:
+            pods = k8s.get(f'{constants.K8S_API_BASE}/namespaces/{namespace}/'
+                           f'pods?fieldSelector=status.phase=Running,'
+                           f'status.podIP={pod_ip}')
+        else:
+            pods = k8s.get(f'{constants.K8S_API_BASE}/'
+                           f'pods?fieldSelector=status.phase=Running,'
+                           f'status.podIP={pod_ip}')
+    except exceptions.K8sClientException:
+        LOG.exception('Error retrieving Pod with IP %s', pod_ip)
+        raise
+    if pods.get('items'):
+        # Only one Pod should have the IP
+        return pods['items'][0]
+    return pod
+
+
+def get_current_endpoints_target(ep, port, spec_ports, ep_name):
+    """Retrieve details about one specific Endpoint target
+
+    Defines the details about the Endpoint target, such as the
+    target address, name, port value and the Pool ID. In case,
+    the Endpoints has no targetRef defined, the name of the
+    target will be the same as the Endpoint.
+
+    :param ep: Endpoint on the Endpoints object
+    :param port: Endpoint port
+    :param spec_ports: dict of port name associated to pool ID
+    :param ep_name: Name of the Endpoints object
+    :returns: Tuple with target address, target name, port number
+              and pool ID.
+    """
+    target_ref = ep.get('targetRef', {})
+    pod_name = ep_name
+    # NOTE(maysams): As we don't support dual-stack, we assume
+    # only one address is possible on the addresses field.
+    address = ep['addresses'][0]
+    if target_ref:
+        pod_name = target_ref.get('name', '')
+    return (address, pod_name, port['port'],
+            spec_ports.get(port.get('name')))
+
+
+def get_subnet_by_ip(nodes_subnets, target_ip):
+    ip = ipaddress.ip_address(target_ip)
+    for nodes_subnet in nodes_subnets:
+        if ip in ipaddress.ip_network(nodes_subnet[1]):
+            return nodes_subnet
+
+    return None
